import 'package:ui_sdk/props/StandardScreenResponse.dart';
import 'package:ui_sdk/props/ApiResponse.dart';
import 'package:ui_sdk/utils/pubsub/localpubsub.dart';
import 'package:flutter/material.dart';
import 'Renderer.dart';
import 'utils/init_data.dart';

abstract class StandardPage {
  late StatefulWidget page;

  dynamic getData();
  Set<String?> getActions();
  dynamic getPageCallBackForAction();
  dynamic getPageContext();

  StandardPage([InitData? args]) {
    page = FlutterPage(
      getData(),
      getActions(), // set of strings
      getPageCallBackForAction(),
      /*
        ........................Syntax..............................

        getPageCallBackForAction() {
          return callback();
        }

        void callback() {

        }
      */
      getPageContext(),
      /*
        getPageContext() {
          return pageReference;
        }
      */
      args: args,
    );
  }

  StatefulWidget getPage() {
    return page;
  }
}

class FlutterPage extends StatefulWidget {
  dynamic apiData;
  Set<String?> actionList;
  dynamic fun;
  dynamic reference;
  Map<String?, Subscription?>? subArray;

  InitData? args;

  FlutterPage(this.apiData, this.actionList, this.fun, this.reference,
      {Key? key, this.args})
      : super(key: key);

  // FlutterPage(
  //     this.apiData, this.actionList, this.fun, this.reference, {this.args},
  //     {Key? key})
  //     : super(key: key);

  @override
  State<FlutterPage> createState() => FlutterPageState();
}

class FlutterPageState extends State<FlutterPage> {
  Renderer renderer = Renderer();

  @override
  void initState() {
    super.initState();
    registerEvents();
  }

  registerEvents() {
    //we have to unsubscribe the elements here
    //but for now we don't have to do right now
    widget.subArray = Map();
    PubSub pubsub = PubSub();
    print("*********************events registering******************");
    // print("janu");
    // print("rashid ${widget.reference}");
    //   print("entering");
    // print(widget.reference);
    //actionlist array

    //set
    widget.actionList.forEach(
      (element) {
        Subscription? sub;
        sub = pubsub.subscribe(element, widget.reference);

        // print(widget.subArray);
        widget.subArray![element] = sub;
        //  print(widget.subArray);

        sub?.stream?.listen(
          (uikAction) {
            // print(uikAction);
            // print(pubsub.stack.top());

            //because the last reference will always be of Prateek
            //   print("sub");
            //   print(" ${widget.reference}  ${pubsub.stack.top()} ");

            if (widget.reference == pubsub.stack.top()) {
              widget.fun(uikAction);
            }
          },
        );
      },
    );
    // print(widget.subArray);
    //   print("widget.reference ${widget.reference}");
    pubsub.stack.push(widget.reference);
    // print(pubsub.stack.length);
    //print(widget.reference);
  }

  @override
  Widget build(BuildContext context) {
    //extracting args
    final args = ModalRoute.of(context)!.settings.arguments ?? null;
    print("args ${args}");

    return WillPopScope(
      onWillPop: () async {
        print("entering in will pop scope");
        // print("Lavesh ${widget.reference}");
        PubSub pubsub = PubSub();
        print("***********************************************");
        if (pubsub.stack.length > 0) {
          print(" before pop ${pubsub.stack.length} -  ${pubsub.stack.top()}");
        }
        pubsub.stack.pop();
        if (pubsub.stack.length > 0) {
          print(" after pop ${pubsub.stack.length} - ${pubsub.stack.top}");
        }
        print("************************************************");
        pubsub.unsubscribe(widget.subArray);
        return true;
      },
      child: MaterialApp(
        debugShowCheckedModeBanner: false,
        title: 'Fetch Data Example',
        theme: ThemeData(
          primarySwatch: Colors.blue,
        ),
        home: Scaffold(
          body: Center(
            child: FutureBuilder<ApiResponse>(
              future: widget.apiData(args),
              builder: (context, snapshot) {
                if (snapshot.hasData) {
                  return renderer.getListWidgetUi(
                    StandardScreenResponse.fromJson(
                      snapshot.requireData.data,
                    ),
                  );
                } else if (snapshot.hasError) {
                  return Text('${snapshot.error}');
                }

                // By default, show a loading spinner.
                return const CircularProgressIndicator(
                  color: Colors.yellow,
                );
              },
            ),
          ),
        ),
      ),
    );
  }
}
