import 'package:ui_sdk/components/UikWidget.dart';
import 'package:ui_sdk/components/WidgetType.dart';
import 'package:ui_sdk/props/UikTextProps.dart';
import 'package:flutter/material.dart';

import '../props/UikTextFieldProps.dart';

class UikTextField extends UikWidget {
  WidgetType _widgetType;
  UikTextFieldProps _uikTextFieldProps;
  UikTextField(this._widgetType, this._uikTextFieldProps)
      : super(_widgetType, _uikTextFieldProps);

  @override
  _UikTextFieldState createState() => _UikTextFieldState(_uikTextFieldProps);
}

class _UikTextFieldState extends State<UikTextField> {
  UikTextFieldProps _uikTextFieldProps;
  _UikTextFieldState(this._uikTextFieldProps);

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: EdgeInsets.fromLTRB(
          _uikTextFieldProps.leftMargin,
          _uikTextFieldProps.topMargin,
          _uikTextFieldProps.rightMargin,
          _uikTextFieldProps.bottomMargin),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Container(
            width: _uikTextFieldProps.width,
            height: _uikTextFieldProps.height,
            decoration: BoxDecoration(
              color: Color(0xffF5F5F5),
              border: (_uikTextFieldProps.error == true)
                  ? Border.all(color: Color(0xffEF5350))
                  : Border.all(color: Color(0xffF5F5F5)),
              borderRadius: BorderRadius.circular(8),
            ),
            child: Row(
              //crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                if (_uikTextFieldProps.leftElement != null) ...[
                  _buildTrailingIcon(_uikTextFieldProps.leftElement)
                ],
                Flexible(
                  child: Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Container(
                        margin: EdgeInsets.fromLTRB(16, 9.5, 16, 0),
                        child: (_uikTextFieldProps.labelText != null)
                            ? Text(
                                _uikTextFieldProps.labelText!,
                                textAlign: TextAlign.start,
                                style: TextStyle(
                                    color: Color(0xff9E9E9E), fontSize: 14),
                              )
                            : null,
                      ),
                      Container(
                        margin: EdgeInsets.fromLTRB(16, 0, 16, 0),
                        child: TextField(
                          cursorHeight: 20,
                          cursorColor: Colors.black,
                          style: TextStyle(fontSize: 16),
                          decoration: InputDecoration(
                            border: InputBorder.none,
                            hintText: _uikTextFieldProps.hintText,
                            //fillColor: Colors.redAccent,
                            isDense: true,
                            contentPadding:
                                (_uikTextFieldProps.labelText == null)
                                    ? EdgeInsets.symmetric(vertical: 10)
                                    : EdgeInsets.symmetric(vertical: 5),
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
                if (_uikTextFieldProps.rightElement != null) ...[
                  _buildLeadingIcon(_uikTextFieldProps.rightElement!),
                ],
              ],
            ),
          ),
          Container(
              margin: EdgeInsets.fromLTRB(16, 8, 0, 0),
              child: Text(
                (_uikTextFieldProps.description != null)
                    ? _uikTextFieldProps.description
                    : (""),
                style: TextStyle(
                  color: (_uikTextFieldProps.error == true)
                      ? Color(0xffEF5350)
                      : Color(0xff9E9E9E),
                ),
              )),
        ],
      ),
    );
  }
}

Widget _buildTrailingIcon(Widget? leftElement) {
  if (leftElement != null) {
    return Row(
      children: <Widget>[
        SizedBox(width: 15),
        //Spacer(),
        leftElement,
      ],
    );
  }
  return Container();
}

Widget _buildLeadingIcon(Widget? rightElement) {
  if (rightElement != null) {
    return Row(
      //crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        rightElement,
        // SizedBox(width: 22),
      ],
    );
  }
  return Container();
}
